@@ -24,6 +24,7 @@ class TNFW(LensProfileBase):
 
         :param interpol: bool, if True, interpolates the functions F(), g() and h()
         """
+        self._s = 0.001
         super(LensProfileBase, self).__init__()
 
     def function(self, x, y, Rs, alpha_Rs, r_trunc, center_x=0, center_y=0):
@@ -38,13 +39,14 @@ class TNFW(LensProfileBase):
         :return:
         """
         rho0_input = self._alpha2rho0(alpha_Rs=alpha_Rs, Rs=Rs)
-        if Rs < 0.0001:
-            Rs = 0.0001
+        #if Rs < 0.0001:
+        #    Rs = 0.0001
         x_ = x - center_x
         y_ = y - center_y
         R = np.sqrt(x_ ** 2 + y_ ** 2)
+        R = np.maximum(R, self._s * Rs)
         f_ = self.nfwPot(R, Rs, rho0_input, r_trunc)
-        #TODO: truncated NFW potential not in place yet!
+
         return f_
 
     def L(self, x, tau):
@@ -54,7 +56,7 @@ class TNFW(LensProfileBase):
         :param tau: t/Rs
         :return:
         """
-
+        x = np.maximum(x, self._s)
         return np.log(x * (tau + np.sqrt(tau ** 2 + x ** 2)) ** -1)
 
     def F(self, x):
@@ -63,18 +65,23 @@ class TNFW(LensProfileBase):
         :param x: r/Rs
         :return:
         """
+        x = np.maximum(x, self._s)
         if isinstance(x, np.ndarray):
-            nfwvals = np.ones_like(x)
-            inds1 = np.where(x < 1)
+            nfwvals = np.zeros_like(x)
+            inds1 = np.where((x < 1) & (x > 0))
             inds2 = np.where(x > 1)
+            inds3 = np.where(x == 0)
             nfwvals[inds1] = (1 - x[inds1] ** 2) ** -.5 * np.arctanh((1 - x[inds1] ** 2) ** .5)
             nfwvals[inds2] = (x[inds2] ** 2 - 1) ** -.5 * np.arctan((x[inds2] ** 2 - 1) ** .5)
+            #nfwvals[inds3] = 0
             return nfwvals
 
         elif isinstance(x, float) or isinstance(x, int):
             if x == 1:
                 return 1
-            if x < 1:
+            elif x == 0:
+                return 0
+            elif x < 1:
                 return (1 - x ** 2) ** -.5 * np.arctanh((1 - x ** 2) ** .5)
             else:
                 return (x ** 2 - 1) ** -.5 * np.arctan((x ** 2 - 1) ** .5)
@@ -82,11 +89,12 @@ class TNFW(LensProfileBase):
     def derivatives(self, x, y, Rs=None, alpha_Rs=None, r_trunc=None, center_x=0, center_y=0):
 
         rho0_input = self._alpha2rho0(alpha_Rs=alpha_Rs, Rs=Rs)
-        if Rs < 0.0000001:
-            Rs = 0.0000001
+        #if Rs < 0.0000001:
+        #    Rs = 0.0000001
         x_ = x - center_x
         y_ = y - center_y
         R = np.sqrt(x_ ** 2 + y_ ** 2)
+        R = np.maximum(R, self._s * Rs)
         f_x, f_y = self.nfwAlpha(R, Rs, rho0_input, r_trunc, x_, y_)
         return f_x, f_y
 
@@ -98,11 +106,12 @@ class TNFW(LensProfileBase):
         returns Hessian matrix of function d^2f/dx^2, d^f/dy^2, d^2/dxdy
         """
         rho0_input = self._alpha2rho0(alpha_Rs=alpha_Rs, Rs=Rs)
-        if Rs < 0.0001:
-            Rs = 0.0001
+        #if Rs < 0.0001:
+        #    Rs = 0.0001
         x_ = x - center_x
         y_ = y - center_y
         R = np.sqrt(x_ ** 2 + y_ ** 2)
+        R = np.maximum(R, self._s * Rs)
 
         kappa = self.density_2d(x_, y_, Rs, rho0_input, r_trunc)
         gamma1, gamma2 = self.nfwGamma(R, Rs, rho0_input, r_trunc, x_, y_)
@@ -127,7 +136,7 @@ class TNFW(LensProfileBase):
 
     def density_2d(self, x, y, Rs, rho0, r_trunc, center_x=0, center_y=0):
         """
-        projected two dimenstional NFW profile (kappa*Sigma_crit)
+        projected two dimensional NFW profile (kappa*Sigma_crit)
 
         :param R: radius of interest
         :type R: float/numpy array
@@ -158,7 +167,7 @@ class TNFW(LensProfileBase):
         """
 
         x = R * Rs ** -1
-
+        x = np.maximum(x, self._s)
         func = (r_trunc ** 2 * (-2 * x * (1 + r_trunc ** 2) + 4 * (1 + x) * r_trunc * np.arctan(x / r_trunc) -
                                 2 * (1 + x) * (-1 + r_trunc ** 2) * np.log(Rs) + 2 * (1 + x) * (-1 + r_trunc ** 2) * np.log(Rs * (1 + x)) +
                                 2 * (1 + x) * (-1 + r_trunc ** 2) * np.log(Rs * r_trunc) -
@@ -180,6 +189,7 @@ class TNFW(LensProfileBase):
         :return: Epsilon(R) projected density at radius R
         """
         x = R / Rs
+        x = np.maximum(x, self._s)
         tau = float(r_trunc) / Rs
         hx = self._h(x, tau)
         return 2 * rho0 * Rs ** 3 * hx
@@ -200,12 +210,14 @@ class TNFW(LensProfileBase):
         :type axis: same as R
         :return: Epsilon(R) projected density at radius R
         """
-        if isinstance(R, int) or isinstance(R, float):
-            R = max(R, 0.00001)
-        else:
-            R[R <= 0.00001] = 0.00001
+        R = np.maximum(R, self._s * Rs)
+        #if isinstance(R, int) or isinstance(R, float):
+        #    R = max(R, 0.00001)
+        #else:
+        #    R[R <= 0.00001] = 0.00001
 
         x = R / Rs
+        x = np.maximum(x, self._s)
         tau = float(r_trunc) / Rs
         gx = self._g(x, tau)
         a = 4 * rho0 * Rs * gx / x ** 2
@@ -229,12 +241,14 @@ class TNFW(LensProfileBase):
         :return: Epsilon(R) projected density at radius R
         """
         c = 0.000001
-        if isinstance(R, int) or isinstance(R, float):
-            R = max(R, c)
-        else:
-            R[R <= c] = c
-
+        #if isinstance(R, int) or isinstance(R, float):
+        #    R = max(R, c)
+        #else:
+        #    R[R <= c] = c
+        R = np.maximum(R, self._s * Rs)
         x = R / Rs
+        #x = np.maximum(x, self._s)
+        #R = np.maximum(R, self._s * Rs)
 
         tau = float(r_trunc) * Rs ** -1
 
@@ -256,6 +270,7 @@ class TNFW(LensProfileBase):
         """
 
         x = R / Rs
+        x = np.maximum(x, self._s)
         tau = r_trunc / Rs
         gx = self._g(x,tau)
         m_2d = 4 * rho0 * Rs * R ** 2 * gx / x ** 2 * np.pi
@@ -271,7 +286,7 @@ class TNFW(LensProfileBase):
         """
         t2 = tau ** 2
         #Fx = self.F(X)
-
+        X = np.maximum(X, self._s )
         _F = self.F(X)
         a = t2*(t2+1)**-2
         if isinstance(X, np.ndarray):
@@ -302,99 +317,67 @@ class TNFW(LensProfileBase):
         :param x: R/Rs
         :type x: float >0
         """
+        x = np.maximum(x, self._s)
         return tau ** 2 * (tau ** 2 + 1) ** -2 * (
                 (tau ** 2 + 1 + 2 * (x ** 2 - 1)) * self.F(x) + tau * np.pi + (tau ** 2 - 1) * np.log(tau) +
                 np.sqrt(tau ** 2 + x ** 2) * (-np.pi + self.L(x, tau) * (tau ** 2 - 1) * tau ** -1))
 
-    def _h(self, X, tau):
+    @staticmethod
+    def _cos_function(x):
+
+        if isinstance(x, np.ndarray) or isinstance(x, list):
+            out = np.empty_like(x)
+            inds1 = np.where(x < 1)
+            inds2 = np.where(x >= 1)
+
+            out[inds1] = -np.arccosh(1 / x[inds1]) ** 2
+            out[inds2] = np.arccos(1 / x[inds2]) ** 2
+
+        elif isinstance(x, float) or isinstance(x, int):
+            if x < 1:
+                out = -np.arccosh(1 / x) ** 2
+            else:
+                out = np.arccos(1 / x) ** 2
+
+        else:
+            raise Exception('x data type '+type(x)+' not recognized.')
+
+        return out
+
+    def _h(self, x, tau):
 
         """
-        a horrible expression for the integral to compute potential
+        expression for the integral to compute potential
 
         :param x: R/Rs
-        :param tau: t/Rs
+        :param tau: r_trunc/Rs
         :type x: float >0
         """
+        x = np.maximum(x, self._s)
 
-        def cos_func(y):
-            if isinstance(y, float) or isinstance(y, int):
-                if y > 1:
-                    return np.arccosh(y)
-                else:
-                    return np.arccos(y)
-            else:
-                values = np.ones_like(y)
-                inds1 = np.where(y < 1)
-                inds2 = np.where(y > 1)
-                values[inds1] = np.arccos(y[inds1])
-                # values[inds2] = np.arccosh(y[inds2])
-                values[inds2] = np.arccos((1 - y[inds2]) ** .5)
-                return values
+        u = x ** 2
+        t2 = tau ** 2
+        Lx = self.L(x, tau)
+        Fx = self.F(x)
 
-        def nfw_func(x):
-            if isinstance(x,float) or isinstance(x,int):
-                if x<1:
-                    return np.log(x / 2.) ** 2 - np.arccosh(1. / x) ** 2
-                else:
-                    return np.log(x / 2.)**2 - np.arccos(1. /x)**2
-            else:
-                inds1 = np.where(x < 1)
-                inds2 = np.where(x >= 1)
-                y = np.zeros_like(x)
-                y[inds1] = np.log(x[inds1] / 2.) ** 2 - np.arccosh(1. / x[inds1]) ** 2
-                y[inds2] = np.log(x[inds2] / 2.) ** 2 - np.arccosh(1. / x[inds2]) ** 2
-
-                return y
-
-        def tnfw_func(x,tau):
-
-            u = x**2
-            t2 = tau**2
-            Lx = self.L(u**.5, tau)
-
-            return (t2 + 1) ** -2 * (
+        return (t2 + 1) ** -2 * (
                 2 * t2 * np.pi * (tau - (t2 + u) ** .5 + tau * np.log(tau + (t2 + u) ** .5))
                 +
                 2 * (t2 - 1) * tau * (t2 + u) ** .5 * Lx
                 +
                 t2 * (t2 - 1) * Lx ** 2
                 +
-                4 * t2 * (u - 1) * self.F(u**.5)
+                4 * t2 * (u - 1) * Fx
                 +
-                t2 * (t2 - 1) * (cos_func(u ** -0.5)) ** 2
+                t2 * (t2 - 1) * self._cos_function(x)
                 +
                 t2 * ((t2 - 1) * np.log(tau) - t2 - 1) * np.log(u)
                 -
                 t2 * (
-                (t2 - 1) * np.log(tau) * np.log(4 * tau) + 2 * np.log(0.5 * tau) - 2 * tau * (tau - np.pi) * np.log(
+                        (t2 - 1) * np.log(tau) * np.log(4 * tau) + 2 * np.log(0.5 * tau) - 2 * tau * (
+                            tau - np.pi) * np.log(
                     tau * 2)))
 
-        c = 1e-9
-        rescale = 1
-
-        if np.any(X-c<1):
-
-            warnings.warn('Truncated NFW potential not yet implemented for x<1. Using the expression for the NFW '
-                          'potential in this regime isntead.')
-            rescale = tnfw_func(1.00001,tau)*nfw_func(1)**-1
-
-        if isinstance(X,float) or isinstance(X,int):
-            if X<1:
-                X = min(X,0.001)
-                return nfw_func(X)*rescale
-            else:
-                return tnfw_func(X,tau)
-        else:
-
-            X[np.where(X<0.001)] = 0.001
-
-            values = np.zeros_like(X)
-            inds1,inds2 = np.where(X<1),np.where(X>=1)
-            values[inds1] = nfw_func(X[inds1])*rescale
-            values[inds2] = tnfw_func(X[inds2],tau)
-
-            return values
-
     def _alpha2rho0(self, alpha_Rs, Rs):
         """
         convert angle at Rs into rho0; neglects the truncation
