@@ -1,6 +1,6 @@
 import numpy as np
 import lenstronomy.Util.util as util
-import lenstronomy.Util.mask as mask_util
+import lenstronomy.Util.mask_util as mask_util
 import lenstronomy.Util.param_util as param_util
 
 
@@ -41,7 +41,7 @@ class LensModelExtensions(object):
             from lenstronomy.LightModel.Profiles.gaussian import Gaussian
             quasar = Gaussian()
         elif shape == 'TORUS':
-            import lenstronomy.LightModel.Profiles.torus as quasar
+            import lenstronomy.LightModel.Profiles.ellipsoid as quasar
         else:
             raise ValueError("shape %s not valid for finite magnification computation!" % shape)
         x_grid, y_grid = util.make_grid(numPix=grid_number, deltapix=deltaPix, subgrid_res=1)
@@ -65,7 +65,7 @@ class LensModelExtensions(object):
 
             betax, betay = self._lensModel.ray_shooting(xcoord + ra, ycoord + dec, kwargs_lens)
 
-            I_image = quasar.function(betax, betay, 1., source_sigma, source_sigma, center_x, center_y)
+            I_image = quasar.function(betax, betay, 1., source_sigma, center_x, center_y)
             mag_finite[i] = np.sum(I_image) * deltaPix**2
         return mag_finite
 
@@ -88,22 +88,24 @@ class LensModelExtensions(object):
             from lenstronomy.LightModel.Profiles.gaussian import Gaussian
             quasar = Gaussian()
         elif shape == 'TORUS':
-            import lenstronomy.LightModel.Profiles.torus as quasar
+            import lenstronomy.LightModel.Profiles.ellipsoid as quasar
         else:
             raise ValueError("shape %s not valid for finite magnification computation!" % shape)
         x_grid, y_grid = util.make_grid(numPix=grid_number, deltapix=deltaPix, subgrid_res=1)
         center_x, center_y = self._lensModel.ray_shooting(x_pos, y_pos, kwargs_lens)
         betax, betay = self._lensModel.ray_shooting(x_grid + x_pos, y_grid + y_pos, kwargs_lens)
-        image = quasar.function(betax, betay, 1., source_sigma, source_sigma, center_x, center_y)
+        image = quasar.function(betax, betay, 1., source_sigma, center_x, center_y)
         return util.array2image(image)
 
     def critical_curve_tiling(self, kwargs_lens, compute_window=5, start_scale=0.5, max_order=10):
         """
 
-        :param kwargs_lens:
-        :param compute_window:
-        :param tiling_scale:
-        :return:
+        :param kwargs_lens: lens model keyword argument list
+        :param compute_window: total window in the image plane where to search for critical curves
+        :param start_scale: float, angular scale on which to start the tiling from (if there are two distinct curves in
+         a region, it might only find one.
+        :param max_order: int, maximum order in the tiling to compute critical curve triangles
+        :return: list of positions representing coordinates of the critical curve (in RA and DEC)
         """
         numPix = int(compute_window / start_scale)
         x_grid_init, y_grid_init = util.make_grid(numPix, deltapix=start_scale, subgrid_res=1)
@@ -138,6 +140,7 @@ class LensModelExtensions(object):
         :param edge2: [ra_coord, dec_coord, magnification]
         :param edge_90: [ra_coord, dec_coord, magnification]
         :param max_order: maximal order to fold triangle
+        :param kwargs_lens: lens model keyword argument list
         :return:
         """
         ra_1, dec_1, mag_1 = edge1
@@ -202,144 +205,7 @@ class LensModelExtensions(object):
         plt.cla()
         return ra_crit_list, dec_crit_list, ra_caustic_list, dec_caustic_list
 
-    def effective_einstein_radius(self, kwargs_lens_list, k=None,
-                                  spacing=1000, get_precision=False, verbose=True):
-        """
-        computes the radius with mean convergence=1
-
-        :param kwargs_lens:
-        :param spacing: number of annular bins to compute the convergence (resolution of the Einstein radius estimate)
-        :param get_precision: If `True`, return the precision of estimated Einstein radius
-        :return:
-        """
-        if 'center_x' in kwargs_lens_list[0]:
-            center_x = kwargs_lens_list[0]['center_x']
-            center_y = kwargs_lens_list[0]['center_y']
-        elif self._lensModel.lens_model_list[0] in ['INTERPOL', 'INTERPOL_SCALED']:
-            center_x, center_y = 0, 0
-        else:
-            center_x, center_y = 0, 0
-        numPix = 200
-        deltaPix = 0.05
-        x_grid, y_grid = util.make_grid(numPix=numPix, deltapix=deltaPix)
-        x_grid += center_x
-        y_grid += center_y
-        kappa = self._lensModel.kappa(x_grid, y_grid, kwargs_lens_list, k=k)
-        if self._lensModel.lens_model_list[0] in ['INTERPOL', 'INTERPOL_SCALED']:
-            center_x = x_grid[kappa == np.max(kappa)]
-            center_y = y_grid[kappa == np.max(kappa)]
-        kappa = util.array2image(kappa)
-        r_array = np.linspace(0.0001, numPix*deltaPix/2., spacing)
-        for r in r_array:
-            mask = np.array(1 - mask_util.mask_center_2d(center_x, center_y, r, x_grid, y_grid))
-            sum_mask = np.sum(mask)
-            if sum_mask > 0:
-                kappa_mean = np.sum(kappa*mask)/np.sum(mask)
-                if kappa_mean < 1:
-                    if get_precision:
-                        return r, r_array[1] - r_array[0]
-                    else:
-                        return r
-        if verbose:
-            print(kwargs_lens_list, "Warning, no Einstein radius computed!")
-        return np.nan #r_array[-1]
-
-    def external_lensing_effect(self, kwargs_lens, lens_model_internal_bool=None):
-        """
-        computes deflection, shear and convergence at (0,0) for those part of the lens model not included in the main deflector
-
-        :param kwargs_lens:
-        :return:
-        """
-        alpha0_x, alpha0_y = 0, 0
-        kappa_ext = 0
-        shear1, shear2 = 0, 0
-        if lens_model_internal_bool is None:
-            lens_model_internal_bool = [True] * len(kwargs_lens)
-        for i, kwargs in enumerate(kwargs_lens):
-            if not lens_model_internal_bool[i] is True:
-                f_x, f_y = self._lensModel.alpha(0, 0, kwargs_lens, k=i)
-                f_xx, f_xy, f_yx, f_yy = self._lensModel.hessian(0, 0, kwargs_lens, k=i)
-                alpha0_x += f_x
-                alpha0_y += f_y
-                kappa_ext += (f_xx + f_yy)/2.
-                shear1 += 1./2 * (f_xx - f_yy)
-                shear2 += f_xy
-        return alpha0_x, alpha0_y, kappa_ext, shear1, shear2
-
-    def external_shear(self, kwargs_lens_list, foreground=False):
-        """
-
-        :param kwargs_lens_list:
-        :return:
-        """
-        for i, model in enumerate(self._lensModel.lens_model_list):
-            if foreground is True:
-                shear_model = 'FOREGROUND_SHEAR'
-            else:
-                shear_model = 'SHEAR'
-            if model == shear_model:
-                e1 = kwargs_lens_list[i]['e1']
-                e2 = kwargs_lens_list[i]['e2']
-                phi, gamma = param_util.ellipticity2phi_gamma(e1, e2)
-                return phi, gamma
-        return 0, 0
-
-    def lens_center(self, kwargs_lens, k=None, bool_list=None, numPix=200, deltaPix=0.01, center_x_init=0, center_y_init=0):
-        """
-        computes the convergence weighted center of a lens model
-
-        :param kwargs_lens: lens model keyword argument list
-        :param bool_list: bool list (optional) to include certain models or not
-        :return: center_x, center_y
-        """
-        x_grid, y_grid = util.make_grid(numPix=numPix, deltapix=deltaPix)
-        x_grid += center_x_init
-        y_grid += center_y_init
-
-        if bool_list is None:
-            kappa = self._lensModel.kappa(x_grid, y_grid, kwargs_lens, k=k)
-        else:
-            kappa = np.zeros_like(x_grid)
-            for k in range(len(kwargs_lens)):
-                if bool_list[k] is True:
-                    kappa += self._lensModel.kappa(x_grid, y_grid, kwargs_lens, k=k)
-        center_x = x_grid[kappa == np.max(kappa)]
-        center_y = y_grid[kappa == np.max(kappa)]
-        return center_x, center_y
-
-    def profile_slope(self, kwargs_lens_list, lens_model_internal_bool=None, num_points=10, verbose=True):
-        """
-        computes the logarithmic power-law slope of a profile
-
-        :param kwargs_lens_list: lens model keyword argument list
-        :param lens_model_internal_bool: bool list, indicate which part of the model to consider
-        :param num_points: number of estimates around the Einstein radius
-        :return:
-        """
-        theta_E = self.effective_einstein_radius(kwargs_lens_list, verbose=verbose)
-        if np.isnan(theta_E):
-            if verbose:
-                print("Could not compute effective slope, because of Einstein radius")
-            return np.nan
-        x0 = kwargs_lens_list[0]['center_x']
-        y0 = kwargs_lens_list[0]['center_y']
-        x, y = util.points_on_circle(theta_E, num_points)
-        dr = 0.01
-        x_dr, y_dr = util.points_on_circle(theta_E + dr, num_points)
-        if lens_model_internal_bool is None:
-            lens_model_internal_bool = [True]*len(kwargs_lens_list)
-
-        alpha_E_x_i, alpha_E_y_i = self._lensModel.alpha(x0 + x, y0 + y, kwargs_lens_list, k=lens_model_internal_bool)
-        alpha_E_r = np.sqrt(alpha_E_x_i**2 + alpha_E_y_i**2)
-        alpha_E_dr_x_i, alpha_E_dr_y_i = self._lensModel.alpha(x0 + x_dr, y0 + y_dr, kwargs_lens_list,
-                                                               k=lens_model_internal_bool)
-        alpha_E_dr = np.sqrt(alpha_E_dr_x_i ** 2 + alpha_E_dr_y_i ** 2)
-        slope = np.mean(np.log(alpha_E_dr / alpha_E_r) / np.log((theta_E + dr) / theta_E))
-        gamma = -slope + 2
-        return gamma
-
-    def hessian_eigenvectors(self, x, y, kwargs_lens):
+    def hessian_eigenvectors(self, x, y, kwargs_lens, diff=None):
         """
         computes magnification eigenvectors at position (x, y)
 
@@ -348,7 +214,7 @@ class LensModelExtensions(object):
         :param kwargs_lens: lens model keyword arguments
         :return: radial stretch, tangential stretch
         """
-        f_xx, f_xy, f_yx, f_yy = self._lensModel.hessian(x, y, kwargs_lens)
+        f_xx, f_xy, f_yx, f_yy = self._lensModel.hessian(x, y, kwargs_lens, diff=diff)
         if isinstance(x, int) or isinstance(x, float):
             A = np.array([[1-f_xx, f_xy], [f_yx, 1-f_yy]])
             w, v = np.linalg.eig(A)
@@ -363,71 +229,164 @@ class LensModelExtensions(object):
                 v11[i], v12[i], v21[i], v22[i] = v[0, 0], v[0, 1], v[1, 0], v[1, 1]
         return w1, w2, v11, v12, v21, v22
 
-    def radial_tangential_stretch(self, x, y, kwargs_lens):
+    def radial_tangential_stretch(self, x, y, kwargs_lens, diff=None, ra_0=0, dec_0=0,
+                                  coordinate_frame_definitions=False):
         """
         computes the radial and tangential stretches at a given position
 
         :param x: x-position
         :param y: y-position
         :param kwargs_lens: lens model keyword arguments
+        :param diff: float or None, finite average differential scale
         :return: radial stretch, tangential stretch
         """
-        w0, w1, v11, v12, v21, v22= self.hessian_eigenvectors(x, y, kwargs_lens)
+        w1, w2, v11, v12, v21, v22 = self.hessian_eigenvectors(x, y, kwargs_lens, diff=diff)
+        v_x, v_y = x - ra_0, y - dec_0
+
+        prod_v1 = v_x*v11 + v_y*v12
+        prod_v2 = v_x*v21 + v_y*v22
         if isinstance(x, int) or isinstance(x, float):
-            if w0 > w1:
-                radial_stretch = 1. / w0
-                tangential_stretch = 1. / w1
-                v_rad1, v_rad2 = v11, v12
-                v_tang1, v_tang2 = v21, v22
+            if (coordinate_frame_definitions is True and abs(prod_v1) >= abs(prod_v2)) or (coordinate_frame_definitions is False and w1 >= w2):
+            #if w1 > w2:
+            #if abs(prod_v1) > abs(prod_v2):  # radial vector has larger scalar product to the zero point
+                lambda_rad = 1. / w1
+                lambda_tan = 1. / w2
+                v1_rad, v2_rad = v11, v12
+                v1_tan, v2_tan = v21, v22
+                prod_r = prod_v1
             else:
-                radial_stretch = 1. / w1
-                tangential_stretch = 1. / w0
-                v_rad1, v_rad2 = v21, v22
-                v_tang1, v_tang2 = v11, v12
+                lambda_rad = 1. / w2
+                lambda_tan = 1. / w1
+                v1_rad, v2_rad = v21, v22
+                v1_tan, v2_tan = v11, v12
+                prod_r = prod_v2
+            if prod_r < 0:  # if radial eigenvector points towards the center
+                v1_rad, v2_rad = -v1_rad, -v2_rad
+            if v1_rad * v2_tan - v2_rad * v1_tan < 0:  # cross product defines orientation of the tangential eigenvector
+                v1_tan *= -1
+                v2_tan *= -1
+
         else:
-            radial_stretch, tangential_stretch, v_rad1, v_rad2, v_tang1, v_tang2 = np.empty(len(x), dtype=float), np.empty(len(x), dtype=float), np.empty_like(x), np.empty_like(x), np.empty_like(x), np.empty_like(x)
+            lambda_rad, lambda_tan, v1_rad, v2_rad, v1_tan, v2_tan = np.empty(len(x), dtype=float), np.empty(len(x), dtype=float), np.empty_like(x), np.empty_like(x), np.empty_like(x), np.empty_like(x)
             for i in range(len(x)):
-                if w0[i] > w1[i]:
-                    radial_stretch[i] = 1. / w0[i]
-                    tangential_stretch[i] = 1. / w1[i]
-                    v_rad1[i], v_rad2[i] = v11[i], v12[i]
-                    v_tang1[i], v_tang2[i] = v21[i], v22[i]
+                if (coordinate_frame_definitions is True and abs(prod_v1[i]) >= abs(prod_v2[i])) or (
+                        coordinate_frame_definitions is False and w1[i] >= w2[i]):
+                #if w1[i] > w2[i]:
+                    lambda_rad[i] = 1. / w1[i]
+                    lambda_tan[i] = 1. / w2[i]
+                    v1_rad[i], v2_rad[i] = v11[i], v12[i]
+                    v1_tan[i], v2_tan[i] = v21[i], v22[i]
+                    prod_r = prod_v1[i]
                 else:
-                    radial_stretch[i] = 1. / w1[i]
-                    tangential_stretch[i] = 1. / w0[i]
-                    v_rad1[i], v_rad2[i] = v21[i], v22[i]
-                    v_tang1[i], v_tang2[i] = v11[i], v12[i]
+                    lambda_rad[i] = 1. / w2[i]
+                    lambda_tan[i] = 1. / w1[i]
+                    v1_rad[i], v2_rad[i] = v21[i], v22[i]
+                    v1_tan[i], v2_tan[i] = v11[i], v12[i]
+                    prod_r = prod_v2[i]
+                if prod_r < 0:  # if radial eigenvector points towards the center
+                    v1_rad[i], v2_rad[i] = -v1_rad[i], -v2_rad[i]
+                if v1_rad[i] * v2_tan[i] - v2_rad[i] * v1_tan[i] < 0:  # cross product defines orientation of the tangential eigenvector
+                    v1_tan[i] *= -1
+                    v2_tan[i] *= -1
 
-        return radial_stretch, tangential_stretch, v_rad1, v_rad2, v_tang1, v_tang2
+        return lambda_rad, lambda_tan, v1_rad, v2_rad, v1_tan, v2_tan
 
-    def radial_tangential_differentials(self, x, y, kwargs_lens, delta=0.001):
+    def radial_tangential_differentials(self, x, y, kwargs_lens, center_x=0, center_y=0, smoothing_3rd=0.001,
+                                        smoothing_2nd=None):
         """
         computes the differentials in stretches and directions
 
         :param x: x-position
         :param y: y-position
         :param kwargs_lens: lens model keyword arguments
-        :param delta: finite differential length in units of angle
+        :param center_x: x-coord of center towards which the rotation direction is defined
+        :param center_y: x-coord of center towards which the rotation direction is defined
+        :param smoothing_3rd: finite differential length of third order in units of angle
+        :param smoothing_2nd: float or None, finite average differential scale of Hessian
         :return:
         """
-        radial_stretch, tangential_stretch, v_rad1, v_rad2, v_tang1, v_tang2 = self.radial_tangential_stretch(x, y, kwargs_lens)
+        lambda_rad, lambda_tan, v1_rad, v2_rad, v1_tan, v2_tan = self.radial_tangential_stretch(x, y, kwargs_lens,
+                                                                                                diff=smoothing_2nd,
+                                                                                                ra_0=center_x, dec_0=center_y,
+                                                                                                coordinate_frame_definitions=True)
+        x0 = x - center_x
+        y0 = y - center_y
 
-        dx_tang = x + delta * v_tang1
-        dy_tang = y + delta * v_tang2
-        rad_dt, tang_dt, v_rad1_dt, v_rad2_dt, v_tang1_dt, v_tang2_dt = self.radial_tangential_stretch(dx_tang, dy_tang, kwargs_lens)
+        # computing angle of tangential vector in regard to the defined coordinate center
+        cos_angle = (v1_tan * x0 + v2_tan * y0) / np.sqrt((x0 ** 2 + y0 ** 2) * (v1_tan ** 2 + v2_tan ** 2))# * np.sign(v1_tan * y0 - v2_tan * x0)
+        orientation_angle = np.arccos(cos_angle) - np.pi / 2
 
-        d_tang_d_tang = (tang_dt - tangential_stretch) / delta * np.sign(v_tang1 * y - v_tang2 * x)
-        cos_delta = v_tang1 * v_tang1_dt + v_tang2 * v_tang2_dt / (np.sqrt(v_tang1**2 + v_tang2**2) * np.sqrt(v_tang1_dt**2 + v_tang2_dt**2))
-        arc_cos = np.arccos(np.abs(np.minimum(cos_delta, 1)))
+        # computing differentials in tangential and radial directions
+        dx_tan = x + smoothing_3rd * v1_tan
+        dy_tan = y + smoothing_3rd * v2_tan
+        lambda_rad_dtan, lambda_tan_dtan, v1_rad_dtan, v2_rad_dtan, v1_tan_dtan, v2_tan_dtan = self.radial_tangential_stretch(dx_tan, dy_tan, kwargs_lens, diff=smoothing_2nd,
+                                                                                                                              ra_0=center_x, dec_0=center_y, coordinate_frame_definitions=True)
+        dx_rad = x + smoothing_3rd * v1_rad
+        dy_rad = y + smoothing_3rd * v2_rad
+        lambda_rad_drad, lambda_tan_drad, v1_rad_drad, v2_rad_drad, v1_tan_drad, v2_tan_drad = self.radial_tangential_stretch(
+            dx_rad, dy_rad, kwargs_lens, diff=smoothing_2nd, ra_0=center_x, dec_0=center_y, coordinate_frame_definitions=True)
 
-        d_angle_d_tang = arc_cos / delta
+        # eigenvalue differentials in tangential and radial direction
+        dlambda_tan_dtan = (lambda_tan_dtan - lambda_tan) / smoothing_3rd# * np.sign(v1_tan * y0 - v2_tan * x0)
+        dlambda_tan_drad = (lambda_tan_drad - lambda_tan) / smoothing_3rd# * np.sign(v1_rad * x0 + v2_rad * y0)
+        dlambda_rad_drad = (lambda_rad_drad - lambda_rad) / smoothing_3rd# * np.sign(v1_rad * x0 + v2_rad * y0)
+        dlambda_rad_dtan = (lambda_rad_dtan - lambda_rad) / smoothing_3rd# * np.sign(v1_rad * x0 + v2_rad * y0)
 
-        dx_rad = x + delta * v_rad1
-        dy_rad = y + delta * v_rad2
-        rad_dr, tang_dr, v_rad1_dr, v_rad2_dr, v_tang1_dr, v_tang2_dr = self.radial_tangential_stretch(dx_rad, dy_rad, kwargs_lens)
-        cos_delta = v_rad1 * v_rad1_dr + v_rad2 * v_rad2_dr / (np.sqrt(v_rad1**2 + v_rad2**2) * np.sqrt(v_rad1_dr**2 + v_rad2_dr**2))
+        # eigenvector direction differentials in tangential and radial direction
+        cos_dphi_tan_dtan = v1_tan * v1_tan_dtan + v2_tan * v2_tan_dtan #/ (np.sqrt(v1_tan**2 + v2_tan**2) * np.sqrt(v1_tan_dtan**2 + v2_tan_dtan**2))
+        norm = np.sqrt(v1_tan**2 + v2_tan**2) * np.sqrt(v1_tan_dtan**2 + v2_tan_dtan**2)
+        cos_dphi_tan_dtan /= norm
+        arc_cos_dphi_tan_dtan = np.arccos(np.abs(np.minimum(cos_dphi_tan_dtan, 1)))
+        dphi_tan_dtan = arc_cos_dphi_tan_dtan / smoothing_3rd
 
-        cos_delta = np.minimum(cos_delta, 1)
-        d_angle_d_rad = np.arccos(cos_delta) / delta
-        d_rad_d_rad = (rad_dr - radial_stretch) / delta * np.sign(v_rad1 * x + v_rad2 * y)
-        return radial_stretch, tangential_stretch, d_tang_d_tang, d_angle_d_tang, d_rad_d_rad, d_angle_d_rad
+        cos_dphi_tan_drad = v1_tan * v1_tan_drad + v2_tan * v2_tan_drad  # / (np.sqrt(v1_tan ** 2 + v2_tan ** 2) * np.sqrt(v1_tan_drad ** 2 + v2_tan_drad ** 2))
+        norm = np.sqrt(v1_tan ** 2 + v2_tan ** 2) * np.sqrt(v1_tan_drad ** 2 + v2_tan_drad ** 2)
+        cos_dphi_tan_drad /= norm
+        arc_cos_dphi_tan_drad = np.arccos(np.abs(np.minimum(cos_dphi_tan_drad, 1)))
+        dphi_tan_drad = arc_cos_dphi_tan_drad / smoothing_3rd
+
+        cos_dphi_rad_drad = v1_rad * v1_rad_drad + v2_rad * v2_rad_drad #/ (np.sqrt(v1_rad**2 + v2_rad**2) * np.sqrt(v1_rad_drad**2 + v2_rad_drad**2))
+        norm = np.sqrt(v1_rad**2 + v2_rad**2) * np.sqrt(v1_rad_drad**2 + v2_rad_drad**2)
+        cos_dphi_rad_drad /= norm
+        cos_dphi_rad_drad = np.minimum(cos_dphi_rad_drad, 1)
+        dphi_rad_drad = np.arccos(cos_dphi_rad_drad) / smoothing_3rd
+
+        cos_dphi_rad_dtan = v1_rad * v1_rad_dtan + v2_rad * v2_rad_dtan # / (np.sqrt(v1_rad ** 2 + v2_rad ** 2) * np.sqrt(v1_rad_dtan ** 2 + v2_rad_dtan ** 2))
+        norm = np.sqrt(v1_rad ** 2 + v2_rad ** 2) * np.sqrt(v1_rad_dtan ** 2 + v2_rad_dtan ** 2)
+        cos_dphi_rad_dtan /= norm
+        cos_dphi_rad_dtan = np.minimum(cos_dphi_rad_dtan, 1)
+        dphi_rad_dtan = np.arccos(cos_dphi_rad_dtan) / smoothing_3rd
+
+        return lambda_rad, lambda_tan, orientation_angle, dlambda_tan_dtan, dlambda_tan_drad, dlambda_rad_drad, dlambda_rad_dtan, dphi_tan_dtan, dphi_tan_drad, dphi_rad_drad, dphi_rad_dtan
+
+    def curved_arc_estimate(self, x, y, kwargs_lens, smoothing=None, smoothing_3rd=0.001):
+        """
+        performs the estimation of the curved arc description at a particular position of an arbitrary lens profile
+
+        :param x: float, x-position where the estimate is provided
+        :param y: float, y-position where the estimate is provided
+        :param kwargs_lens: lens model keyword arguments
+        :return: keyword argument list corresponding to a CURVED_ARC profile at (x, y) given the initial lens model
+        """
+        radial_stretch, tangential_stretch, v_rad1, v_rad2, v_tang1, v_tang2 = self.radial_tangential_stretch(x, y, kwargs_lens, diff=smoothing)
+        dx_tang = x + smoothing_3rd * v_tang1
+        dy_tang = y + smoothing_3rd * v_tang2
+        rad_dt, tang_dt, v_rad1_dt, v_rad2_dt, v_tang1_dt, v_tang2_dt = self.radial_tangential_stretch(dx_tang, dy_tang,
+                                                                                                       kwargs_lens,
+                                                                                                       diff=smoothing)
+        d_tang1 = v_tang1_dt - v_tang1
+        d_tang2 = v_tang2_dt - v_tang2
+        delta = np.sqrt(d_tang1**2 + d_tang2**2)
+        if delta > 1:
+            d_tang1 = v_tang1_dt + v_tang1
+            d_tang2 = v_tang2_dt + v_tang2
+            delta = np.sqrt(d_tang1 ** 2 + d_tang2 ** 2)
+        curvature = delta / smoothing_3rd
+        direction = np.arctan2(v_rad2 * np.sign(v_rad1 * x + v_rad2 * y), v_rad1 * np.sign(v_rad1 * x + v_rad2 * y))
+        #direction = np.arctan2(v_rad2, v_rad1)
+        kwargs_arc = {'radial_stretch': radial_stretch,
+                      'tangential_stretch': tangential_stretch,
+                      'curvature': curvature,
+                      'direction': direction,
+                      'center_x': x, 'center_y': y}
+        return kwargs_arc
