@@ -1,4 +1,4 @@
-import numpy as np
+from lenstronomy.Util.util import convert_bool_list
 
 
 class ProfileListBase(object):
@@ -17,6 +17,7 @@ class ProfileListBase(object):
         self.func_list = self._load_model_instances(lens_model_list, custom_class=numerical_alpha_class,
                                                     lens_redshift_list=lens_redshift_list,
                                                     z_source_convention=z_source_convention)
+        self._num_func = len(self.func_list)
         self._model_list = lens_model_list
 
     def _load_model_instances(self, lens_model_list, custom_class=None, lens_redshift_list=None,
@@ -47,8 +48,8 @@ class ProfileListBase(object):
 
         :param lens_type: string, lens model type
         :param custom_class: custom class
-        :param z_lens:
-        :param z_source:
+        :param z_lens: lens redshift  # currently only used in NFW_MC model as this is redshift dependent
+        :param z_source: source redshift  # currently only used in NFW_MC model as this is redshift dependent
         :return: class instance of the lens model type
         """
 
@@ -89,8 +90,8 @@ class ProfileListBase(object):
             from lenstronomy.LensModel.Profiles.nie import NIE
             return NIE()
         elif lens_type == 'NIE_SIMPLE':
-            from lenstronomy.LensModel.Profiles.nie import NIE_simple
-            return NIE_simple()
+            from lenstronomy.LensModel.Profiles.nie import NIEMajorAxis
+            return NIEMajorAxis()
         elif lens_type == 'CHAMELEON':
             from lenstronomy.LensModel.Profiles.chameleon import Chameleon
             return Chameleon()
@@ -124,10 +125,13 @@ class ProfileListBase(object):
         elif lens_type == 'CNFW':
             from lenstronomy.LensModel.Profiles.cnfw import CNFW
             return CNFW()
+        elif lens_type == 'CNFW_ELLIPSE':
+            from lenstronomy.LensModel.Profiles.cnfw_ellipse import CNFW_ELLIPSE
+            return CNFW_ELLIPSE()
         elif lens_type == 'CTNFW_GAUSS_DEC':
             from lenstronomy.LensModel.Profiles.gauss_decomposition import CTNFWGaussDec
             return CTNFWGaussDec()
-        elif lens_type =='NFW_MC':
+        elif lens_type == 'NFW_MC':
             from lenstronomy.LensModel.Profiles.nfw_mass_concentration import NFWMC
             return NFWMC(z_lens=z_lens, z_source=z_source)
         elif lens_type == 'SERSIC':
@@ -191,9 +195,24 @@ class ProfileListBase(object):
         elif lens_type == 'CURVED_ARC':
             from lenstronomy.LensModel.Profiles.curved_arc import CurvedArc
             return CurvedArc()
+        elif lens_type == 'ARC_PERT':
+            from lenstronomy.LensModel.Profiles.arc_perturbations import ArcPerturbations
+            return ArcPerturbations()
         elif lens_type == 'coreBURKERT':
             from lenstronomy.LensModel.Profiles.coreBurkert import CoreBurkert
             return CoreBurkert()
+        elif lens_type == 'CORED_DENSITY':
+            from lenstronomy.LensModel.Profiles.cored_density import CoredDensity
+            return CoredDensity()
+        elif lens_type == 'CORED_DENSITY_2':
+            from lenstronomy.LensModel.Profiles.cored_density_2 import CoredDensity2
+            return CoredDensity2()
+        elif lens_type == 'CORED_DENSITY_MST':
+            from lenstronomy.LensModel.Profiles.cored_density_mst import CoredDensityMST
+            return CoredDensityMST(profile_type='CORED_DENSITY')
+        elif lens_type == 'CORED_DENSITY_2_MST':
+            from lenstronomy.LensModel.Profiles.cored_density_mst import CoredDensityMST
+            return CoredDensityMST(profile_type='CORED_DENSITY_2')
         elif lens_type == 'NumericalAlpha':
             from lenstronomy.LensModel.Profiles.numerical_deflections import NumericalAlpha
             return NumericalAlpha(custom_class)
@@ -205,27 +224,13 @@ class ProfileListBase(object):
         returns a bool list of the length of the lens models
         if k = None: returns bool list with True's
         if k is int, returns bool list with False's but k'th is True
+        if k is a list of int, e.g. [0, 3, 5], returns a bool list with True's in the integers listed and False elsewhere
+        if k is a boolean list, checks for size to match the numbers of models and returns it
 
         :param k: None, int, or list of ints
         :return: bool list
         """
-        n = len(self.func_list)
-        if k is None:
-            bool_list = [True] * n
-        elif isinstance(k, (int, np.integer)):
-            bool_list = [False] * n
-            bool_list[k] = True
-        else:
-            bool_list = [False] * n
-            for i, k_i in enumerate(k):
-                if k_i is not False:
-                    if k_i is True:
-                        bool_list[i] = True
-                    elif k_i < n:
-                        bool_list[k_i] = True
-                    else:
-                        raise ValueError("k as set by %s is not convertable in a bool string!" % k)
-        return bool_list
+        return convert_bool_list(n=self._num_func, k=k)
 
     def set_static(self, kwargs_list):
         """
