@@ -27,8 +27,9 @@ class FittingSequence(object):
         :param kwargs_constraints:
         :param kwargs_likelihood:
         :param kwargs_params:
-        :param mpi:
-        :param verbose: bool, if True
+        :param mpi: MPI option (bool), if True, will launch an MPI Pool job for the steps in the fitting sequence where
+        possible
+        :param verbose: bool, if True prints temporary results and indicators of the fitting process
         """
         self.kwargs_data_joint = kwargs_data_joint
         self.multi_band_list = kwargs_data_joint.get('multi_band_list', [])
@@ -38,13 +39,13 @@ class FittingSequence(object):
                                                      num_bands=len(self.multi_band_list))
         self._mcmc_init_samples = None
 
-    #def kwargs_fixed(self):
-    #    """
-    #    returns the updated kwargs_fixed from the update Manager
+    def kwargs_fixed(self):
+        """
+        returns the updated kwargs_fixed from the update Manager
 
-    #    :return: list of fixed kwargs, see UpdateManager()
-    #    """
-    #    return self._updateManager.fixed_kwargs
+        :return: list of fixed kwargs, see UpdateManager()
+        """
+        return self._updateManager.fixed_kwargs
 
     def fit_sequence(self, fitting_list):
         """
@@ -53,8 +54,6 @@ class FittingSequence(object):
         :return: fitting results
         """
         chain_list = []
-        #param_list = []
-        #samples_mcmc, param_mcmc, dist_mcmc = [], [], []
         for i, fitting in enumerate(fitting_list):
             fitting_type = fitting[0]
             kwargs = fitting[1]
@@ -65,6 +64,9 @@ class FittingSequence(object):
             elif fitting_type == 'update_settings':
                 self.update_settings(**kwargs)
 
+            elif fitting_type == 'set_param_value':
+                self.set_param_value(**kwargs)
+
             elif fitting_type == 'fix_not_computed':
                 self.fix_not_computed(**kwargs)
 
@@ -79,12 +81,19 @@ class FittingSequence(object):
                 self._updateManager.update_param_state(**kwargs_result)
                 chain_list.append([fitting_type, chain, param])
 
+            elif fitting_type == 'SIMPLEX':
+                kwargs_result = self.simplex(**kwargs)
+                self._updateManager.update_param_state(**kwargs_result)
+                chain_list.append([fitting_type, kwargs_result])
+
             elif fitting_type == 'MCMC':
                 if not 'init_samples' in kwargs:
                     kwargs['init_samples'] = self._mcmc_init_samples
                 elif kwargs['init_samples'] is None:
                     kwargs['init_samples'] = self._mcmc_init_samples
                 mcmc_output = self.mcmc(**kwargs)
+                kwargs_result = self._result_from_mcmc(mcmc_output)
+                self._updateManager.update_param_state(**kwargs_result)
                 chain_list.append(mcmc_output)
 
             elif fitting_type == 'nested_sampling':
@@ -92,7 +101,7 @@ class FittingSequence(object):
                 chain_list.append(ns_output)
 
             else:
-                raise ValueError("fitting_sequence %s is not supported. Please use: 'PSO', 'MCMC', 'psf_iteration', "
+                raise ValueError("fitting_sequence %s is not supported. Please use: 'PSO', 'SIMPLEX', 'MCMC', 'psf_iteration', "
                                  "'restart', 'update_settings' or ""'align_images'" % fitting_type)
         return chain_list
 
@@ -159,8 +168,26 @@ class FittingSequence(object):
         likelihoodModule = LikelihoodModule(self.kwargs_data_joint, kwargs_model, self.param_class, **kwargs_likelihood)
         return likelihoodModule
 
+    def simplex(self, n_iterations, method='Nelder-Mead'):
+        """
+        Downhill simplex optimization using the Nelder-Mead algorithm.
+
+        :param n_iterations: maximum number of iterations to perform
+        :param method: the optimization method used, see documentation in scipy.optimize.minimize
+        :return: result of the best fit
+        """
+
+        param_class = self.param_class
+        kwargs_temp = self._updateManager.parameter_state
+        init_pos = param_class.kwargs2args(**kwargs_temp)
+        sampler = Sampler(likelihoodModule=self.likelihoodModule)
+        result = sampler.simplex(init_pos, n_iterations, method)
+
+        kwargs_result = param_class.args2kwargs(result, bijective=True)
+        return kwargs_result
+
     def mcmc(self, n_burn, n_run, walkerRatio, sigma_scale=1, threadCount=1, init_samples=None, re_use_samples=True,
-             sampler_type='EMCEE'):
+             sampler_type='EMCEE', progress=True):
         """
         MCMC routine
 
@@ -172,6 +199,7 @@ class FittingSequence(object):
         :param init_samples: initial sample from where to start the MCMC process
         :param re_use_samples: bool, if True, re-uses the samples described in init_samples.nOtherwise starts from scratch.
         :param sampler_type: string, which MCMC sampler to be used. Options are: 'COSMOHAMMER, and 'EMCEE'
+        :param progress: boolean, if True shows progress bar in EMCEE
         :return: list of output arguments, e.g. MCMC samples, parameter names, logL distances of all samples specified by the specific sampler used
         """
 
@@ -198,7 +226,8 @@ class FittingSequence(object):
 
         if sampler_type is 'EMCEE':
             n_walkers = num_param * walkerRatio
-            samples, dist = mcmc_class.mcmc_emcee(n_walkers, n_run, n_burn, mean_start, sigma_start, mpi=self._mpi)
+            samples, dist = mcmc_class.mcmc_emcee(n_walkers, n_run, n_burn, mean_start, sigma_start, mpi=self._mpi,
+                                                  threadCount=threadCount, progress=progress)
             output = [sampler_type, samples, param_list, dist]
         else:
             raise ValueError('sampler_type %s not supported!' % sampler_type)
@@ -420,6 +449,22 @@ class FittingSequence(object):
         self._updateManager.update_limits(change_source_lower_limit, change_source_upper_limit)
         return 0
 
+    def set_param_value(self, **kwargs):
+        """
+        Set a parameter to a specific value. `kwargs` are below.
+        :param lens: [[i_model, ['param1', 'param2',...], [...]]
+        :type lens:
+        :param source: [[i_model, ['param1', 'param2',...], [...]]
+        :type source:
+        :param lens_light: [[i_model, ['param1', 'param2',...], [...]]
+        :type lens_light:
+        :param ps: [[i_model, ['param1', 'param2',...], [...]]
+        :type ps:
+        :return: 0, the value of the param is overwritten
+        :rtype:
+        """
+        self._updateManager.update_param_value(**kwargs)
+
     def fix_not_computed(self, free_bands):
         """
         fixes lens model parameters of imaging bands/frames that are not computed and frees the parameters of the other
@@ -448,3 +493,18 @@ class FittingSequence(object):
         """
         kwargs_result = self.param_class.args2kwargs(result, bijective=True)
         self._updateManager.update_param_state(**kwargs_result)
+
+    def _result_from_mcmc(self, mcmc_output):
+        """
+
+        :param mcmc_output: list returned by self.mcmc()
+        :return: kwargs_result like returned by self.pso(), from best logL MCMC sample
+        """
+        _, samples, _, logL_values = mcmc_output
+        # get index of best logL sample
+        bestfit_idx = np.argmax(logL_values)
+        bestfit_sample = samples[bestfit_idx, :]
+        bestfit_result = bestfit_sample.tolist()
+        # get corresponding kwargs
+        kwargs_result = self.param_class.args2kwargs(bestfit_result, bijective=True)
+        return kwargs_result
