@@ -55,6 +55,8 @@ class UpdateManager(object):
 
         self._kwargs_temp = self.init_kwargs
 
+    #TODO check compatibility with number of point sources provided as well as other parameter labelings
+
     @property
     def init_kwargs(self):
         return {'kwargs_lens': self._lens_init, 'kwargs_source': self._source_init,
@@ -76,7 +78,7 @@ class UpdateManager(object):
         return self._lens_upper, self._source_upper, self._lens_light_upper, self._ps_upper, self._special_upper, self._extinction_upper
 
     @property
-    def _fixed_kwargs(self):
+    def fixed_kwargs(self):
         return self._lens_fixed, self._source_fixed, self._lens_light_fixed, self._ps_fixed, self._special_fixed, self._extinction_fixed
 
     def set_init_state(self):
@@ -111,26 +113,52 @@ class UpdateManager(object):
     def update_param_state(self, kwargs_lens=None, kwargs_source=None, kwargs_lens_light=None, kwargs_ps=None,
                            kwargs_special=None, kwargs_extinction=None):
         """
-        updates the temporary state of the parameters being saved
+        updates the temporary state of the parameters being saved. ATTENTION: Any previous knowledge gets lost if you
+        call this function
 
         :param kwargs_lens:
         :param kwargs_source:
         :param kwargs_lens_light:
         :param kwargs_ps:
         :param kwargs_special:
+        :param kwargs_extinction:
         :return:
         """
         self._kwargs_temp = {'kwargs_lens': kwargs_lens, 'kwargs_source': kwargs_source,
                              'kwargs_lens_light': kwargs_lens_light, 'kwargs_ps': kwargs_ps,
                              'kwargs_special': kwargs_special, 'kwargs_extinction': kwargs_extinction}
 
+    def update_param_value(self, lens=[], source=[], lens_light=[], ps=[]):
+        """
+        Set a model parameter to a specific value.
+        :param lens: [[i_model, ['param1', 'param2',...], [...]]
+        :type lens:
+        :param source: [[i_model, ['param1', 'param2',...], [...]]
+        :type source:
+        :param lens_light: [[i_model, ['param1', 'param2',...], [...]]
+        :type lens_light:
+        :param ps: [[i_model, ['param1', 'param2',...], [...]]
+        :type ps:
+        :return: 0, the value of the param is overwritten
+        :rtype:
+        """
+        for items, kwargs_key in zip([lens, source, lens_light, ps],
+            ['kwargs_lens', 'kwargs_source', 'kwargs_lens_light', 'kwargs_ps']):
+            for item in items:
+                index = item[0]
+                keys = item[1]
+                values = item[2]
+
+                for key, value in zip(keys, values):
+                    self._kwargs_temp[kwargs_key][index][key] = value
+
     @property
     def param_class(self):
         """
 
         :return: instance of the Param class with the recent options and bounds
         """
-        kwargs_fixed_lens, kwargs_fixed_source, kwargs_fixed_lens_light, kwargs_fixed_ps, kwargs_fixed_special, kwargs_fixed_extinction = self._fixed_kwargs
+        kwargs_fixed_lens, kwargs_fixed_source, kwargs_fixed_lens_light, kwargs_fixed_ps, kwargs_fixed_special, kwargs_fixed_extinction = self.fixed_kwargs
         kwargs_lower_lens, kwargs_lower_source, kwargs_lower_lens_light, kwargs_lower_ps, kwargs_lower_special, kwargs_lower_extinction = self._lower_kwargs
         kwargs_upper_lens, kwargs_upper_source, kwargs_upper_lens_light, kwargs_upper_ps, kwargs_upper_special, kwargs_upper_extinction = self._upper_kwargs
         kwargs_model = self.kwargs_model
@@ -172,10 +200,11 @@ class UpdateManager(object):
         if not change_source_upper_limit is None:
             self._source_upper = self._update_limit(change_source_upper_limit, self._source_upper)
 
-    def _update_limit(self, change_limit, kwargs_limit_previous):
+    @staticmethod
+    def _update_limit(change_limit, kwargs_limit_previous):
         """
 
-        :param change_limit: imput format of def update_limits
+        :param change_limit: input format of def update_limits
         :param kwargs_limit_previous: all limits of a model type
         :return: update limits
         """
@@ -188,17 +217,12 @@ class UpdateManager(object):
                 kwargs_limit_updated[i_model][param_name] = values[j]
         return kwargs_limit_updated
 
-    def update_fixed(self, lens_add_fixed=[],
-                     source_add_fixed=[], lens_light_add_fixed=[], ps_add_fixed=[], special_add_fixed=[], lens_remove_fixed=[],
-                     source_remove_fixed=[], lens_light_remove_fixed=[], ps_remove_fixed=[], special_remove_fixed=[]):
+    def update_fixed(self, lens_add_fixed=[], source_add_fixed=[], lens_light_add_fixed=[], ps_add_fixed=[],
+                     special_add_fixed=[], lens_remove_fixed=[], source_remove_fixed=[], lens_light_remove_fixed=[],
+                     ps_remove_fixed=[], special_remove_fixed=[]):
         """
         adds the values of the keyword arguments that are stated in the _add_fixed to the existing fixed arguments.
 
-        :param kwargs_lens:
-        :param kwargs_source:
-        :param kwargs_lens_light:
-        :param kwargs_ps:
-        :param kwargs_cosmo:
         :param lens_add_fixed:
         :param source_add_fixed:
         :param lens_light_add_fixed:
