@@ -5,7 +5,6 @@ import numpy as np
 import numpy.testing as npt
 
 from lenstronomy.LensModel.lens_model import LensModel
-from lenstronomy.LensModel.numeric_lens_differentials import NumericLens
 
 
 class TestNumerics(object):
@@ -14,32 +13,31 @@ class TestNumerics(object):
     """
     def setup(self):
         self.lensModel = LensModel(['GAUSSIAN'])
-        self.lensModelNum = NumericLens(['GAUSSIAN'])
         self.kwargs = [{'amp': 1./4., 'sigma_x': 2., 'sigma_y': 2., 'center_x': 0., 'center_y': 0.}]
 
     def test_kappa(self):
         x, y = 1., 1.
         output = self.lensModel.kappa(x, y, self.kwargs)
-        output_num = self.lensModelNum.kappa(x, y, self.kwargs)
+        output_num = self.lensModel.kappa(x, y, self.kwargs, diff=0.00001)
         npt.assert_almost_equal(output_num, output, decimal=5)
 
     def test_gamma(self):
         x, y = 1., 2.
         output1, output2 = self.lensModel.gamma(x, y, self.kwargs)
-        output1_num, output2_num = self.lensModelNum.gamma(x, y, self.kwargs)
+        output1_num, output2_num = self.lensModel.gamma(x, y, self.kwargs, diff=0.00001)
         npt.assert_almost_equal(output1_num, output1, decimal=5)
         npt.assert_almost_equal(output2_num, output2, decimal=5)
 
     def test_magnification(self):
         x, y = 1., 1.
         output = self.lensModel.magnification(x, y, self.kwargs)
-        output_num = self.lensModelNum.magnification(x, y, self.kwargs)
+        output_num = self.lensModel.magnification(x, y, self.kwargs, diff=0.00001)
         npt.assert_almost_equal(output_num, output, decimal=5)
 
     def test_differentials(self):
         x, y = 1., 1.
         f_xx, f_xy, f_yx, f_yy = self.lensModel.hessian(x, y, self.kwargs)
-        f_xx_num, f_xy_num, f_yx_num, f_yy_num = self.lensModelNum.hessian(x, y, self.kwargs)
+        f_xx_num, f_xy_num, f_yx_num, f_yy_num = self.lensModel.hessian(x, y, self.kwargs, diff=0.00001)
         assert f_xy_num == f_yx_num
         npt.assert_almost_equal(f_xx_num, f_xx, decimal=5)
         npt.assert_almost_equal(f_xy_num, f_xy, decimal=5)
@@ -47,10 +45,16 @@ class TestNumerics(object):
         npt.assert_almost_equal(f_yy_num, f_yy, decimal=5)
 
     def test_flexion(self):
-        lensModel = NumericLens(lens_model_list=['FLEXION'])
+        lensModel = LensModel(lens_model_list=['FLEXION'])
         g1, g2, g3, g4 = 0.01, 0.02, 0.03, 0.04
         kwargs = [{'g1': g1, 'g2': g2, 'g3': g3, 'g4': g4}]
-        f_xxx, f_xxy, f_xyy, f_yyy = lensModel.flexion(x=1., y=1., kwargs=kwargs)
+        f_xxx, f_xxy, f_xyy, f_yyy = lensModel.flexion(x=1., y=1., kwargs=kwargs, diff=0.0001)
+        npt.assert_almost_equal(f_xxx, g1, decimal=2)
+        npt.assert_almost_equal(f_xxy, g2, decimal=2)
+        npt.assert_almost_equal(f_xyy, g3, decimal=2)
+        npt.assert_almost_equal(f_yyy, g4, decimal=2)
+
+        f_xxx, f_xxy, f_xyy, f_yyy = lensModel.flexion(x=1., y=1., kwargs=kwargs, diff=0.0001, hessian_diff=0.001)
         npt.assert_almost_equal(f_xxx, g1, decimal=2)
         npt.assert_almost_equal(f_xxy, g2, decimal=2)
         npt.assert_almost_equal(f_xyy, g3, decimal=2)
@@ -64,41 +68,27 @@ class TestNumericsProfile(object):
     def setup(self):
         pass
 
-    def assert_differentials(self, lens_model, kwargs):
-        lensModelNum = NumericLens(lens_model)
-        lensModelNum.diff = 0.000001
+    def assert_differentials(self, lens_model, kwargs, potential=True):
+        #lensModelNum = NumericLens(lens_model)
+        diff = 0.000001
         #x, y = 1., 2.
-        x = np.linspace(start=0.1, stop=8, num=10)
+        x = np.linspace(start=0.1, stop=5.5, num=10)
         y = np.zeros_like(x)
         lensModel = LensModel(lens_model)
-        f_x, f_y = lensModel.lens_model.alpha(x, y, [kwargs])
         f_xx, f_xy, f_yx, f_yy = lensModel.hessian(x, y, [kwargs])
-        f_x_num, f_y_num = lensModelNum.lens_model.alpha(x, y, [kwargs])
-        f_xx_num, f_xy_num, f_yx_num, f_yy_num = lensModelNum.hessian(x, y, [kwargs])
-        print(f_xx_num, f_xx)
-        print(f_yy_num, f_yy)
-        print(f_xy_num, f_xy)
-        npt.assert_almost_equal(f_x , f_x_num, decimal=5)
-        npt.assert_almost_equal(f_y , f_y_num, decimal=5)
-        npt.assert_almost_equal(f_xx , f_xx_num, decimal=3)
-        npt.assert_almost_equal(f_yy , f_yy_num, decimal=3)
-        npt.assert_almost_equal(f_xy , f_xy_num, decimal=3)
+        f_xx_num, f_xy_num, f_yx_num, f_yy_num = lensModel.hessian(x, y, [kwargs], diff=diff)
 
-        x, y = 1., 0.
-        f_x, f_y = lensModel.lens_model.alpha(x, y, [kwargs])
-        f_xx, f_xy, f_yx, f_yy = lensModel.hessian(x, y, [kwargs])
-        f_x_num, f_y_num = lensModelNum.lens_model.alpha(x, y, [kwargs])
-        f_xx_num, f_xy_num, f_yx_num, f_yy_num = lensModelNum.hessian(x, y, [kwargs])
-        print(f_xx_num, f_xx)
-        print(f_yy_num, f_yy)
-        print(f_xy_num, f_xy)
-        print(f_xx_num + f_yy_num, f_xx + f_yy)
-        npt.assert_almost_equal(f_x, f_x_num, decimal=5)
-        npt.assert_almost_equal(f_y, f_y_num, decimal=5)
         npt.assert_almost_equal(f_xx, f_xx_num, decimal=3)
         npt.assert_almost_equal(f_yy, f_yy_num, decimal=3)
         npt.assert_almost_equal(f_xy, f_xy_num, decimal=3)
 
+        if potential is True:
+            f_x, f_y = lensModel.alpha(x, y, [kwargs])
+            f_x_num, f_y_num = lensModel.alpha(x, y, [kwargs], diff=diff)
+
+            npt.assert_almost_equal(f_x, f_x_num, decimal=3)
+            npt.assert_almost_equal(f_y, f_y_num, decimal=3)
+
     def test_gaussian(self):
         lens_model = ['GAUSSIAN']
         kwargs = {'amp': 1. / 4., 'sigma_x': 2., 'sigma_y': 2., 'center_x': 0., 'center_y': 0.}
@@ -125,7 +115,7 @@ class TestNumericsProfile(object):
         self.assert_differentials(lens_model, kwargs)
 
     def test_external_shear(self):
-        kwargs = {'e1': 0.1, 'e2': -0.1}
+        kwargs = {'gamma1': 0.1, 'gamma2': -0.1}
         lens_model = ['SHEAR']
         self.assert_differentials(lens_model, kwargs)
 
@@ -154,7 +144,7 @@ class TestNumericsProfile(object):
         lens_model = ['TNFW']
         self.assert_differentials(lens_model, kwargs)
 
-        kwargs = {'Rs': 2, 'alpha_Rs': 1, 'r_trunc': 7}
+        kwargs = {'Rs': 2., 'alpha_Rs': 1., 'r_trunc': 7}
         lens_model = ['TNFW']
         self.assert_differentials(lens_model, kwargs)
 
@@ -240,7 +230,7 @@ class TestNumericsProfile(object):
         self.assert_differentials(lens_model, kwargs)
 
     def test_NIE_simple(self):
-        kwargs = {'theta_E': 2., 'q': 0.3, 's': 0.04}
+        kwargs = {'b': 2., 'q': 0.3, 's': 0.04}
         lens_model = ['NIE_SIMPLE']
         self.assert_differentials(lens_model, kwargs)
 
@@ -252,10 +242,35 @@ class TestNumericsProfile(object):
         self.assert_differentials(lens_model, kwargs)
 
     def test_cnfw(self):
-        kwargs={'Rs':2, 'alpha_Rs': 1, 'r_core':0.3}
+        kwargs={'Rs': 15.5, 'alpha_Rs': 1., 'r_core': 8.}
         lens_model = ['CNFW']
+        self.assert_differentials(lens_model, kwargs, potential=True)
+
+    def test_cnfw_ellipse(self):
+        kwargs = {'alpha_Rs': .1, 'Rs': 5., 'r_core': 0.1, 'e1': 0.04, 'e2': -0.04}
+        lens_model = ['CNFW_ELLIPSE']
+        self.assert_differentials(lens_model, kwargs, potential=True)
+
+    def test_cored_density(self):
+        kwargs = {'sigma0': 0.1, 'r_core': 8}
+        lens_model = ['CORED_DENSITY']
+        self.assert_differentials(lens_model, kwargs)
+
+    def test_cored_density_2(self):
+        kwargs = {'sigma0': 0.1, 'r_core': 8}
+        lens_model = ['CORED_DENSITY_2']
+        self.assert_differentials(lens_model, kwargs)
+
+    def test_cored_density_mst(self):
+        kwargs = {'lambda_approx': 1.1, 'r_core': 8}
+        lens_model = ['CORED_DENSITY_MST']
+        self.assert_differentials(lens_model, kwargs)
+
+    def test_cored_density_2_mst(self):
+        kwargs = {'lambda_approx': 1.1, 'r_core': 8}
+        lens_model = ['CORED_DENSITY_2_MST']
         self.assert_differentials(lens_model, kwargs)
 
 
 if __name__ == '__main__':
-    pytest.main("-k TestLensModel")
\ No newline at end of file
+    pytest.main("-k TestLensModel")
