@@ -8,12 +8,12 @@ from lenstronomy.Sampling.likelihood import LikelihoodModule
 from lenstronomy.Sampling.parameters import Param
 from lenstronomy.LensModel.lens_model import LensModel
 from lenstronomy.LightModel.light_model import LightModel
-from lenstronomy.Sampling.sampler import Sampler
+from lenstronomy.Sampling.sampler import Sampler, choose_pool
 from lenstronomy.Data.imaging_data import ImageData
 from lenstronomy.Data.psf import PSF
 
 
-class TestFittingSequence(object):
+class TestSampler(object):
     """
     test the fitting sequences
     """
@@ -75,12 +75,12 @@ class TestFittingSequence(object):
 
         kwargs_constraints = {'image_plane_source_list': [False] * len(source_model_list)}
 
-        kwargs_likelihood = {
-                                  'source_marg': True,
-                                  'position_uncertainty': 0.004,
-                                  'check_solver': False,
-                                  'solver_tolerance': 0.001,
-                                  }
+        kwargs_likelihood = {'source_marg': False,
+                             'image_position_uncertainty': 0.004,
+                             'check_matched_source_position': False,
+                             'source_position_tolerance': 0.001,
+                             'source_position_sigma': 0.001,
+                             }
         self.param_class = Param(kwargs_model, **kwargs_constraints)
         self.Likelihood = LikelihoodModule(kwargs_data_joint=kwargs_data_joint, kwargs_model=kwargs_model,
                                            param_class=self.param_class, **kwargs_likelihood)
@@ -103,6 +103,22 @@ class TestFittingSequence(object):
         sigma_start = np.ones_like(mean_start) * 0.1
         samples, dist = self.sampler.mcmc_emcee(n_walkers, n_run, n_burn, mean_start, sigma_start, mpi=False)
         assert len(samples) == n_walkers * n_run
+        assert len(dist) == len(samples)
+
+    def test_choose_pool(self):
+        import schwimmbad
+        pool, is_master = choose_pool(mpi=False, processes=1, use_dill=True)
+        assert is_master is True
+        assert isinstance(pool, schwimmbad.serial.SerialPool)
+
+        pool, is_master = choose_pool(mpi=False, processes=2, use_dill=True)
+        assert is_master is True
+        assert isinstance(pool, schwimmbad.multiprocessing.MultiPool)
+
+        #NOTE: MPI cannot be tested here (needs to be launched with mpirun)
+        # pool, is_master = choose_pool(mpi=True, processes=1, use_dill=True)
+        # assert is_master is True
+        # assert isinstance(pool, schwimmbad.mpi.MPIPool)
 
 
 if __name__ == '__main__':
